@@ -1,4 +1,4 @@
-#ident "$Id: lookup_yp.c,v 1.4 2004/05/10 12:44:30 raven Exp $"
+#ident "$Id: lookup_yp.c,v 1.14 2005/02/10 10:59:59 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  lookup_yp.c - module for Linux automountd to access a YP (NIS)
@@ -21,6 +21,7 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <time.h>
+#include <signal.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,16 +44,8 @@ struct lookup_context {
 };
 
 struct callback_data {
-	int status;
-	int map;
 	const char *root;
-	char direct_base[KEY_MAX_LEN + 1];
-	const char *name;
-	int name_len;
-	unsigned long type;
-	const char *mapname;
 	time_t age;
-	struct lookup_context *context;
 };
 
 int lookup_version = AUTOFS_LOOKUP_VERSION;	/* Required by protocol */
@@ -92,7 +85,9 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 int yp_all_callback(int status, char *ypkey, int ypkeylen,
 		    char *val, int vallen, char *ypcb_data)
 {
-	time_t *age = (time_t *) ypcb_data;
+	struct callback_data *cbdata = (struct callback_data *) ypcb_data;
+	const char *root = cbdata->root;
+	time_t age = cbdata->age;
 	char *key;
 	char *mapent;
 
@@ -107,20 +102,23 @@ int yp_all_callback(int status, char *ypkey, int ypkeylen,
 	strncpy(mapent, val, vallen);
 	*(mapent + vallen) = '\0';
 
-	cache_update(key, mapent, *age);
+	cache_add(root, key, mapent, age);
 
 	return 0;
 }
 
-static int read_map(const char *root, struct lookup_context *context)
+static int read_map(const char *root, time_t age, struct lookup_context *context)
 {
 	struct lookup_context *ctxt = (struct lookup_context *) context;
 	struct ypall_callback ypcb;
-	time_t age = time(NULL);
+	struct callback_data ypcb_data;
 	int err;
 
+	ypcb_data.root = root;
+	ypcb_data.age = age;
+
 	ypcb.foreach = yp_all_callback;
-	ypcb.data = (char *) &age;
+	ypcb.data = (char *) &ypcb_data;
 
 	err = yp_all((char *) ctxt->domainname, (char *) ctxt->mapname, &ypcb);
 
@@ -136,13 +134,14 @@ static int read_map(const char *root, struct lookup_context *context)
 	return 1;
 }
 
-int lookup_ghost(const char *root, int ghost, void *context)
+int lookup_ghost(const char *root, int ghost, time_t now, void *context)
 {
 	struct lookup_context *ctxt = (struct lookup_context *) context;
+	time_t age = now ? now : time(NULL);
 	struct mapent_cache *me;
 	int status = 1;
 
-	if (!read_map(root, ctxt))
+	if (!read_map(root, age, ctxt))
 		return LKP_FAIL;
 
 	status = cache_ghost(root, ghost, ctxt->mapname, "yp", ctxt->parse);
@@ -162,90 +161,143 @@ int lookup_ghost(const char *root, int ghost, void *context)
 	return status;
 }
 
+static int lookup_one(const char *root,
+		      const char *key, int key_len,
+		      struct lookup_context *ctxt)
+{
+	char *mapent;
+	int mapent_len;
+	time_t age = time(NULL);
+	int err;
+
+	/*
+	 * For reasons unknown, the standard YP definitions doesn't
+	 * define input strings as const char *.  However, my
+	 * understanding is that they will not be modified by the
+	 * library.
+	 */
+	err = yp_match((char *) ctxt->domainname, (char *) ctxt->mapname,
+		       (char *) key, key_len, &mapent, &mapent_len);
+
+	if (err != YPERR_SUCCESS) {
+		if (err == YPERR_KEY)
+			return CHE_MISSING;
+
+		return -err;
+	}
+
+	return cache_update(root, key, mapent, age);
+}
+
+static int lookup_wild(const char *root, struct lookup_context *ctxt)
+{
+	char *mapent;
+	int mapent_len;
+	time_t age = time(NULL);
+	int err;
+
+	mapent = alloca(MAPENT_MAX_LEN + 1);
+	if (!mapent)
+		return 0;
+
+	err = yp_match((char *) ctxt->domainname,
+		       (char *) ctxt->mapname, "*", 1, &mapent, &mapent_len);
+
+	if (err != YPERR_SUCCESS) {
+		if (err == YPERR_KEY)
+			return CHE_MISSING;
+
+		return -err;
+	}
+
+	return cache_update(root, "*", mapent, age);
+}
+
 int lookup_mount(const char *root, const char *name, int name_len, void *context)
 {
 	struct lookup_context *ctxt = (struct lookup_context *) context;
 	char key[KEY_MAX_LEN + 1];
 	int key_len;
-	char *mapent = NULL;
-	struct mapent_cache *me = NULL;
-	time_t age = time(NULL);
+	char *mapent;
 	int mapent_len;
-	int err, rv;
+	struct mapent_cache *me;
+	time_t now = time(NULL);
+	time_t t_last_read;
+	int need_hup = 0;
+	int ret;
 
 	debug(MODPREFIX "looking up %s", name);
 
-	me = cache_lookup(name);
-	if (me == NULL)
-		if (sprintf(key, "%s/%s", root, name))
-			me = cache_lookup(key);
+	if (ap.type == LKP_DIRECT)
+		key_len = snprintf(key, KEY_MAX_LEN, "%s/%s", root, name);
+	else
+		key_len = snprintf(key, KEY_MAX_LEN, "%s", name);
 
-	if (me == NULL) {
+	if (key_len > KEY_MAX_LEN)
+		return 1;
+
+	/* check map and if change is detected re-read map */
+	ret = lookup_one(root, key, key_len, ctxt);
+	if (!ret)
+		return 1;
+
+	debug("ret = %d", ret);
+
+	if (ret < 0) {
+		warn(MODPREFIX 
+		     "lookup for %s failed: %s", name, yperr_string(-ret));
+		return 1;
+	}
+
+	me = cache_lookup_first();
+	t_last_read = me ? now - me->age : ap.exp_runfreq + 1;
+
+	if (t_last_read > ap.exp_runfreq)
+		if (ret & (CHE_UPDATED | CHE_MISSING))
+			need_hup = 1;
+
+	if (ret == CHE_MISSING) {
+		int wild = CHE_MISSING;
+
+		/* Maybe update wild card map entry */
+		if (ap.type == LKP_INDIRECT) {
+			wild = lookup_wild(root, ctxt);
+			if (wild == CHE_MISSING)
+				cache_delete(root, "*", 0);
+		}
+
+		if (cache_delete(root, key, 0) &&
+				wild & (CHE_MISSING | CHE_FAIL))
+			rmdir_path(key);
+	}
+
+
+	me = cache_lookup(key);
+	if (me) {
+		mapent = alloca(strlen(me->mapent) + 1);
+		mapent_len = sprintf(mapent, "%s", me->mapent);
+	} else {
 		/* path component, do submount */
 		me = cache_partial_match(key);
-
 		if (me) {
-			mapent = malloc(strlen(ctxt->mapname) + 20);
+			mapent = alloca(strlen(ctxt->mapname) + 20);
 			mapent_len =
 			    sprintf(mapent, "-fstype=autofs yp:%s", ctxt->mapname);
 		}
-	} else {
-		mapent = malloc(strlen(me->mapent) + 1);
-		mapent_len = sprintf(mapent, me->mapent);
 	}
 
-	if (!me) {
-		/* For reasons unknown, the standard YP definitions doesn't
-		   define input strings as const char *.  However, my
-		   understanding is that they will not be modified by the
-		   library. */
-		err = yp_match((char *) ctxt->domainname, (char *) ctxt->mapname,
-			       (char *) name, name_len, &mapent, &mapent_len);
-
-		if (err != YPERR_SUCCESS) {
-			if (err != YPERR_KEY)
-				goto out_err;
-
-			/* See if there is an entry "root/name" */
-			key_len = sprintf(key, "%s/%s", root, name);
-			err = yp_match((char *) ctxt->domainname, 
-				       (char *) ctxt->mapname,
-				       key, key_len, &mapent, &mapent_len);
-
-			if (err != YPERR_SUCCESS) {
-				if (err != YPERR_KEY)
-					goto out_err;
-				/* 
-				 * Try to get the "*" entry if there is one i
-				 * - note that we *don't* modify "name" so
-				 *   & -> the name we used, not "*"
-				 */
-				err =
-				    yp_match((char *) ctxt->domainname,
-					     (char *) ctxt->mapname, "*", 1, 
-					     &mapent, &mapent_len);
-			} else
-				cache_update(key, mapent, age);
-
-			if (err)
-				goto out_err;
-		} else
-			cache_update(name, mapent, age);
+	if (me) {
+		mapent[mapent_len] = '\0';
+		debug(MODPREFIX "%s -> %s", key, mapent);
+		ret = ctxt->parse->parse_mount(root, name, name_len,
+						mapent, ctxt->parse->context);
 	}
 
-	mapent[mapent_len] = '\0';
+	/* Have parent update its map */
+	if (need_hup)
+		kill(getppid(), SIGHUP);
 
-	debug(MODPREFIX "%s -> %s", name, mapent);
-
-	rv = ctxt->parse->parse_mount(root, name, name_len, mapent, ctxt->parse->context);
-	free(mapent);
-	return rv;
-
-out_err:
-	warn(MODPREFIX "lookup for %s failed: %s", name, yperr_string(err));
-	if (mapent)
-		free(mapent);
-	return 1;
+	return ret;
 }
 
 int lookup_done(void *context)
