@@ -1,4 +1,4 @@
-#ident "$Id: mount_nfs.c,v 1.12 2004/05/18 12:20:08 raven Exp $"
+#ident "$Id: mount_nfs.c,v 1.22 2005/04/05 12:42:42 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  * mount_nfs.c - Module for Linux automountd to mount an NFS filesystem,
@@ -39,8 +39,6 @@
 
 int mount_version = AUTOFS_MOUNT_VERSION;	/* Required by protocol */
 
-extern struct autofs_point ap;
-
 static int udpproto;
 static short port_discard;
 
@@ -118,7 +116,6 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 {
 	char *p = what;
 	char *winner = NULL;
-	char *is_replicated = NULL;
 	int winner_weight = INT_MAX, local = 0;
 	double winner_time = 0;
 	char *delim;
@@ -130,15 +127,6 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 		return -1;
 	}
 
-	/*
-	 * If it's not a replicated server map entry we need
-	 * to only check for a local mount and return the mount
-	 * string
-	 */
-	is_replicated = strpbrk(p, "(,");
-	if (skiplocal)
-		return local;
-
 	while (p && *p) {
 		char *next;
 		unsigned int ping_stat = 0;
@@ -180,9 +168,9 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 		} else
 			break;
 
-		/* p points to a server, next is our next parse point */
+		/* p points to a server, "next is our next parse point */
 		if (!skiplocal) {
-			/* First, check if it's up and if it's localhost */
+			/* Check if it's localhost */
 			struct hostent *he;
 			char **haddr;
 
@@ -216,21 +204,23 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 				break;
 		}
 
-		/*
-		 * If it's not local and it's a replicated server map entry
-		 * is it alive
-		 */
-		if (!local && is_replicated && !(ping_stat = rpc_ping(p, sec, micros))) {
+		/* ping each (or the) entry to see if it's alive. */
+		ping_stat = rpc_ping(p, sec, micros);
+		if (!ping_stat) {
 			p = next;
 			continue;
 		}
 
-		/* see if we have a previous 'winner' */
+		/* First unweighted or only host is alive so set winner */
 		if (!winner) {
 			winner = p;
+			/* No more to check, return it */
+			if (!next || !*next)
+				break;
 		}
-		/* compare RPC times if there are no weighted hosts */
-		else if (winner_weight == INT_MAX) {
+
+		/* Multiple entries and no weighted hosts so compare times */
+		if (winner_weight == INT_MAX) {
 			int status;
 			double resp_time;
 			unsigned int vers = NFS2_VERSION;
@@ -241,13 +231,14 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 				proto = ping_stat & 0xff00;
 			}
 
-			status = rpc_time(winner, vers, proto, sec, micros, &resp_time);
+			status = rpc_time(p, vers, proto, sec, micros, &resp_time);
 			/* did we time the first winner? */
 			if (winner_time == 0) {
-				if (status)
+				if (status) {
+					winner = p;
 					winner_time = resp_time;
-				else
-					winner_time = 6;
+				} else
+					winner_time = 501;
 			} else {
 				if ((status) && (resp_time < winner_time)) {
 					winner = p;
@@ -261,23 +252,18 @@ int get_best_mount(char *what, const char *original, int longtimeout, int skiplo
 	debug(MODPREFIX "winner = %s local = %d", winner, local);
 
 	/*
-	 * We didn't find a weighted winner or local and it's a replicated
-	 * server map entry
+	 * We didn't find a weighted winner or local
 	 */
-	if (!local && is_replicated && winner_weight == INT_MAX) {
+	if (!local && winner_weight == INT_MAX) {
 		/* We had more than one contender and none responded in time */
-		if (winner_time != 0 && winner_time > 5) {
+		if (winner_time != 0 && winner_time > 500) {
 			/* We've already tried a longer timeout */
-			if (longtimeout) {
-				/* SOL: Just pick the first one */
-				winner = what;
-			}
-			/* Reset string and try again */
-			else {
+			if (!longtimeout) {
+				/* Reset string and try again */
 				strcpy(what, original);
 
 				debug(MODPREFIX 
-				      "all hosts rpc timed out for '%s', "
+				      "all hosts timed out for '%s', "
 				      "retrying with longer timeout",
 				      original);
 
@@ -333,8 +319,9 @@ int mount_mount(const char *root, const char *name, int name_len,
 	char *nfsoptions = NULL;
 	int local, err;
 	int nosymlink = 0;
+	int ro = 0;            /* Set if mount bind should be read-only */
 
-	debug(MODPREFIX " root=%s name=%s what=%s, fstype=%s, options=%s",
+	debug(MODPREFIX "root=%s name=%s what=%s, fstype=%s, options=%s",
 	      root, name, what, fstype, options);
 
 	whatstr = alloca(strlen(what) + 1);
@@ -387,14 +374,18 @@ int mount_mount(const char *root, const char *name, int name_len,
 			if (strncmp("nosymlink", cp, end - cp + 1) == 0)
 				nosymlink = 1;
 			else {
+				/* Check for options that also make sense
+				   with bind mounts */
+				if (strncmp("ro", cp, end - cp + 1) == 0)
+					ro = 1;
 				/* and jump over trailing white space */
 				memcpy(nfsp, cp, comma - cp + 1);
 				nfsp += comma - cp + 1;
 			}
 		}
 
-		debug(MODPREFIX "nfs options=\"%s\", nosymlink=%d",
-		      nfsoptions, nosymlink);
+		debug(MODPREFIX "nfs options=\"%s\", nosymlink=%d, ro=%d",
+		      nfsoptions, nosymlink, ro);
 	}
 
 	local = 0;
@@ -426,46 +417,54 @@ int mount_mount(const char *root, const char *name, int name_len,
 	if (local) {
 		/* Local host -- do a "bind" */
 
+		const char *bind_options = ro ? "ro" : "";
+
 		debug(MODPREFIX "%s is local, doing bind", name);
 
 		return mount_bind->mount_mount(root, name, name_len,
-			       whatstr, "bind", NULL, mount_bind->context);
+					       whatstr, "bind", bind_options,
+					       mount_bind->context);
 	} else {
 		/* Not a local host - do an NFS mount */
-		int status;
+		int status, existed = 1;
 
 		debug(MODPREFIX "calling mkdir_path %s", fullpath);
-		if ((status = mkdir_path(fullpath, 0555)) && errno != EEXIST) {
+
+		status = mkdir_path(fullpath, 0555);
+		if (status && errno != EEXIST) {
 			error(MODPREFIX "mkdir_path %s failed: %m", fullpath);
 			return 1;
 		}
 
-		if (is_mounted(fullpath)) {
-			error("BUG: %s already mounted", fullpath);
+		if (!status)
+			existed = 0;
+
+		if (is_mounted(_PATH_MOUNTED, fullpath)) {
+			error(MODPREFIX 
+			  "warning: %s is already mounted", fullpath);
 			return 0;
 		}
 
-		wait_for_lock();
 		if (nfsoptions && *nfsoptions) {
 			debug(MODPREFIX "calling mount -t nfs " SLOPPY 
 			      " -o %s %s %s", nfsoptions, whatstr, fullpath);
 
-			err = spawnl(LOG_NOTICE, MOUNTED_LOCK,
+			err = spawnll(LOG_NOTICE,
 				     PATH_MOUNT, PATH_MOUNT, "-t",
 				     "nfs", SLOPPYOPT "-o", nfsoptions,
 				     whatstr, fullpath, NULL);
 		} else {
 			debug(MODPREFIX "calling mount -t nfs %s %s",
 			      whatstr, fullpath);
-			err = spawnl(LOG_NOTICE, MOUNTED_LOCK,
+			err = spawnll(LOG_NOTICE,
 				     PATH_MOUNT, PATH_MOUNT, "-t",
 				     "nfs", whatstr, fullpath, NULL);
 		}
-		unlink(AUTOFS_LOCK);
 
 		if (err) {
-			if (!ap.ghost && name_len)
+			if ((!ap.ghost && name_len) || !existed)
 				rmdir_path(name);
+
 			error(MODPREFIX "nfs: mount failure %s on %s",
 			      whatstr, fullpath);
 			return 1;
