@@ -1,4 +1,4 @@
-#ident "$Id: parse_sun.c,v 1.10 2004/05/18 12:22:40 raven Exp $"
+#ident "$Id: parse_sun.c,v 1.28 2005/04/05 12:42:42 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  parse_sun.c - module for Linux automountd to parse a Sun-format
@@ -6,6 +6,7 @@
  * 
  *   Copyright 1997 Transmeta Corporation - All Rights Reserved
  *   Copyright 2000 Jeremy Fitzhardinge <jeremy@goop.org>
+ *   Copyright 2004, 2005 Ian Kent <raven@themaw.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -55,6 +56,13 @@ struct parse_context {
 	int slashify_colons;	/* Change colons to slashes? */
 };
 
+struct multi_mnt {
+	char *path;
+	char *options;
+	char *location;
+	struct multi_mnt *next;
+};
+
 struct utsname un;
 char processor[65];		/* Not defined on Linux, so we make our own */
 
@@ -70,6 +78,7 @@ static struct substvar
 
 /* Default context pattern */
 
+static char *child_args = NULL;
 static struct parse_context default_context = {
 	NULL,			/* No mount options */
 	&sv_osvers,		/* The substvar predefined variables */
@@ -252,6 +261,23 @@ const char *skipspace(const char *whence)
 	}
 }
 
+/*
+ * Check a string to see if a colon appears before the next '/'.
+ */
+int check_colon(const char *str)
+{
+	char *ptr = (char *) str;
+
+	while (*ptr && *ptr != ':' && *ptr != '/') {
+		ptr++;
+	}
+
+	if (!*ptr || *ptr == '/')
+		return 0;
+
+	return 1;
+}
+
 /* Get the length of a chunk delimitered by whitespace */
 int chunklen(const char *whence, int expect_colon)
 {
@@ -261,9 +287,12 @@ int chunklen(const char *whence, int expect_colon)
 	for (; *whence; whence++, n++) {
 		switch (*whence) {
 		case '\\':
-			quote = 1;
-			continue;
-
+			if( quote ) {
+				break;
+			} else {
+				quote = 1;
+				continue;
+			}
 		case ':':
 			if (expect_colon)
 				expect_colon = 0;
@@ -376,7 +405,19 @@ int parse_init(int argc, const char *const *argv, void **context)
 						*(sv->val++) = '\0';
 					else
 						sv->val = "";
-
+					/* we use 5 for the "-D", the "=", and the null */
+					if (child_args) {
+						child_args = realloc(child_args, strlen(child_args) + strlen(sv->def) + strlen(sv->val) + 5);
+						strcat(child_args, ",");
+					}
+					else { /* No comma, so only +4 */
+						child_args = malloc(strlen(sv->def) + strlen(sv->val) + 4);
+						*child_args = '\0';
+					}
+					strcat(child_args, "-D");
+					strcat(child_args, sv->def);
+					strcat(child_args, "=");
+					strcat(child_args, sv->val);
 					sv->next = ctxt->subst;
 					ctxt->subst = sv;
 				}
@@ -391,7 +432,7 @@ int parse_init(int argc, const char *const *argv, void **context)
 					bval = 1;
 				}
 
-				if (strmcmp(xopt, "slashify-colons", 1))
+				if (!strmcmp(xopt, "slashify-colons", 1))
 					ctxt->slashify_colons = bval;
 				else
 					error(MODPREFIX "unknown option: %s",
@@ -451,13 +492,17 @@ static char *dequote(const char *str, int strlen)
 	char *cp = ret;
 	const char *scp;
 	int origlen = strlen;
+	int quote = 0;
 
 	if (ret == NULL)
 		return NULL;
 
 	for (scp = str; strlen > 0 && *scp; scp++, strlen--) {
-		if (*scp == '\\')
+		if (*scp == '\\' && !quote ) {
+			quote = 1;
 			continue;
+		}
+		quote = 0;
 		*cp++ = *scp;
 	}
 	*cp = '\0';
@@ -560,12 +605,40 @@ static int sun_mount(const char *root, const char *name, int namelen,
 				np += comma - cp + 1;
 			}
 		}
-		if (np > noptions)
-			np[-1] = '\0';
+
+		if (np > noptions + len) {
+			warn(MODPREFIX "options string truncated");
+			np[len] = '\0';
+		} else
+			*(np - 1) = '\0';
 
 		options = noptions;
 	}
 
+
+	if (child_args && !strcmp(fstype, "autofs")) {
+		char *noptions;
+
+		if (!options) {
+			noptions = alloca(strlen(child_args) + 1);
+			*noptions = '\0';
+		} else {
+			noptions = alloca(strlen(options) + strlen(child_args) + 2);
+
+			if (noptions) {
+				strcpy(noptions, options);
+				strcat(noptions, ",");
+			}
+		}
+
+		if (noptions) {
+			strcat(noptions, child_args);
+			options = noptions;
+		} else {
+			error(MODPREFIX "alloca failed for options");
+		}
+	}
+
 	while (*name == '/') {
 		name++;
 		namelen--;
@@ -578,11 +651,25 @@ static int sun_mount(const char *root, const char *name, int namelen,
 	memcpy(what, loc, loclen);
 	what[loclen] = '\0';
 
-	if (! strcmp(fstype, "autofs") && strchr(loc, ':') == NULL) {
-		what = alloca(loclen + 3 + 1); /* 1 for '0', 3 for yp: */
-		memcpy(what, "yp:", 3);
-		memcpy(what + 3, loc, loclen);
-		what[loclen + 3] = '\0';
+	if (!strcmp(fstype, "autofs") && strchr(loc, ':') == NULL) {
+		char mtype[7];
+		int mtype_len;
+
+		if (loc[0] == '/') {
+			mtype_len = 5;
+			if (loc[1] == '/')
+				strcpy(mtype, "ldap:");
+			else
+				strcpy(mtype, "file:");
+		} else {
+			mtype_len = 3;
+			strcpy(mtype, "yp:");
+		}
+
+		what = alloca(loclen + mtype_len + 1);
+		memcpy(what, mtype, mtype_len);
+		memcpy(what + mtype_len, loc, loclen);
+		what[loclen + mtype_len] = '\0';
 	} else {
 		what = alloca(loclen + 1);
 		memcpy(what, loc, loclen);
@@ -602,14 +689,119 @@ static int sun_mount(const char *root, const char *name, int namelen,
 			      options);
 	}
 
-	if (nonstrict && rv) {
-		debug("ignoring failure of non-strict mount");
-		return 0;
-	}
+	if (nonstrict && rv)
+		return -rv;
 
 	return rv;
 }
 
+/*
+ * Build list of mounts in shortest -> longest order.
+ * Pass in list head and return list head.
+ */
+struct multi_mnt *multi_add_list(struct multi_mnt *list,
+				 char *path, char *options, char *location)
+{
+	struct multi_mnt *mmptr, *new, *old = NULL;
+	int plen;
+
+	if (!path || !options || !location)
+		return NULL;
+
+	new = malloc(sizeof(struct multi_mnt));
+	if (!new)
+		return NULL;
+
+	new->path = path;
+	new->options = options;
+	new->location = location;
+
+	plen = strlen(path);
+	mmptr = list;
+	while (mmptr) {
+		if (plen <= strlen(mmptr->path))
+			break;
+		old = mmptr;
+		mmptr = mmptr->next;
+	}
+
+	if (old)
+		old->next = new;
+	new->next = mmptr;
+
+	return old ? list : new;
+}
+
+void multi_free_list(struct multi_mnt *list)
+{
+	struct multi_mnt *next;
+
+	if (!list)
+		return;
+
+	next = list;
+	while (next) {
+		struct multi_mnt *this = next;
+
+		next = this->next;
+
+		if (this->path)
+			free(this->path);
+
+		if (this->options)
+			free(this->options);
+
+		if (this->location)
+			free(this->location);
+
+		free(this);
+	}
+}
+
+/*
+ * Scan map entry looking for evidence it has multiple key/mapent
+ * pairs.
+ */
+static int check_is_multi(const char *mapent)
+{
+	const char *p = (char *) mapent;
+	int multi = 0;
+	int first_chunk = 0;
+
+	while (*p) {
+		p = skipspace(p);
+
+		/*
+		 * After the first chunk there can be additional
+		 * locations (possibly not multi) or possibly an
+		 * options string if the first entry includes the
+		 * optional '/' (is multi). Following this any
+		 * path that begins with '/' indicates a mutil-mount
+		 * entry.
+		 */
+		if (first_chunk) {
+			if (*p == '/' || *p == '-') {
+				multi = 1;
+				break;
+			}
+		}
+
+		while (*p == '-') {
+			p += chunklen(p, 0);
+			p = skipspace(p);
+		}
+
+		/*
+		 * Expect either a path or location
+		 * after which it's a multi mount.
+		 */
+		p += chunklen(p, check_colon(p));
+		first_chunk++;
+	}
+
+	return multi;
+}
+
 /*
  * syntax is:
  *	[-options] location [location] ...
@@ -620,7 +812,7 @@ int parse_mount(const char *root, const char *name,
 {
 	struct parse_context *ctxt = (struct parse_context *) context;
 	char *pmapent, *options;
-	const char *p, *q;
+	const char *p;
 	int mapent_len, rv;
 	int optlen;
 
@@ -630,6 +822,7 @@ int parse_mount(const char *root, const char *name,
 		error(MODPREFIX "alloca: %m");
 		return 1;
 	}
+	pmapent[mapent_len] = '\0';
 
 	expandsunent(mapent, pmapent, name, ctxt->subst, ctxt->slashify_colons);
 
@@ -662,9 +855,10 @@ int parse_mount(const char *root, const char *name,
 
 	debug(MODPREFIX "gathered options: %s", options);
 
-	if (*p == '/') {
-		int l;
+	if (check_is_multi(p)) {
+		struct multi_mnt *list, *head = NULL;
 		char *multi_root;
+		int l;
 
 		multi_root = alloca(strlen(root) + name_len + 2);
 		if (!multi_root) {
@@ -681,16 +875,22 @@ int parse_mount(const char *root, const char *name,
 		do {
 			char *myoptions = strdup(options);
 			char *path, *loc;
-			int pathlen, loclen;
 
 			if (myoptions == NULL) {
 				error(MODPREFIX "multi strdup: %m");
 				free(options);
+				multi_free_list(head);
 				return 1;
 			}
 
 			path = dequote(p, l = chunklen(p, 0));
-			pathlen = strlen(path);
+			if (!path) {
+				error(MODPREFIX "out of memory");
+				free(myoptions);
+				free(options);
+				multi_free_list(head);
+				return 1;
+			}
 
 			p += l;
 			p = skipspace(p);
@@ -708,79 +908,151 @@ int parse_mount(const char *root, const char *name,
 						    "multi concat_options: %m");
 						free(options);
 						free(path);
+						multi_free_list(head);
 						return 1;
 					}
 					p = skipspace(p);
 				} while (*p == '-');
 			}
 
-			q = p;
-			while (*q && *q != '/') {
-				l = chunklen(q, 1);
-				q += l;
-				q = skipspace(q);
-			}
-			l = q - p;
+			/* Skip over colon escape */
+			if (*p == ':')
+				p++;
 
-			loc = dequote(p, l);
-			loclen = strlen(loc);
-
-			if (loc == NULL || path == NULL) {
+			loc = dequote(p, l = chunklen(p, check_colon(p)));
+			if (!loc) {
 				error(MODPREFIX "out of memory");
-				free(loc);
 				free(path);
+				free(myoptions);
 				free(options);
+				multi_free_list(head);
 				return 1;
 			}
 
 			p += l;
 			p = skipspace(p);
 
+			while (*p && *p != '/') {
+				char *ent;
+
+				ent = dequote(p, l = chunklen(p, check_colon(p)));
+				if (!ent) {
+					error(MODPREFIX "out of memory");
+					free(path);
+					free(myoptions);
+					free(options);
+					multi_free_list(head);
+					return 1;
+				}
+
+				loc = realloc(loc, strlen(loc) + l + 2);
+				if (!loc) {
+					error(MODPREFIX "out of memory");
+					free(ent);
+					free(path);
+					free(myoptions);
+					free(options);
+					multi_free_list(head);
+					return 1;
+				}
+
+				strcat(loc, " ");
+				strcat(loc, ent);
+
+				free(ent);
+
+				p += l;
+				p = skipspace(p);
+			}
+
+			list = head;
+			head = multi_add_list(list, path, myoptions, loc);
+			if (!head) {
+				free(loc);
+				free(path);
+				free(options);
+				free(myoptions);
+				multi_free_list(list);
+				return 1;
+			}
+		} while (*p == '/');
+
+		list = head;
+		while (list) {
 			debug(MODPREFIX
 			      "multimount: %.*s on %.*s with options %s",
-			      loclen, loc, pathlen, path, myoptions);
+			      strlen(list->location), list->location,
+			      strlen(list->path), list->path, list->options);
 
-			rv = sun_mount(multi_root, path, pathlen, loc, loclen,
-				       myoptions);
-			free(path);
-			free(loc);
-			free(myoptions);
+			rv = sun_mount(multi_root,
+				       list->path, strlen(list->path),
+				       list->location, strlen(list->location),
+				       list->options);
 
-			if (rv)
+			/* Convert non-strict failure into success */
+			if (rv < 0) {
+				rv = 0;
+				debug("parse_mount: ignoring failure of non-strict mount");
+			} else if (rv > 0)
 				break;
 
-		} while (*p == '/');
+			list = list->next;
+		}
+
+		multi_free_list(head);
 
 		free(options);
 		return rv;
 	} else {
 		/* Normal (non-multi) entries */
 		char *loc;
-		int loclen, l;
+		int loclen;
+		int l;
 
 		if (*p == ':')
 			p++;	/* Sun escape for entries starting with / */
 
-		q = p;
-		while (*q) {
-			l = chunklen(q, 1);
-			q += l;
-			q = skipspace(q);
-		}
-		l = q - p;
-
-		loc = dequote(p, l);
-		loclen = strlen(loc);
-
-		if (loc == NULL) {
+		loc = dequote(p, l = chunklen(p, check_colon(p)));
+		if (!loc) {
 			error(MODPREFIX "out of memory");
-			free(loc);
 			free(options);
 			return 1;
 		}
 
+		p += l;
+		p = skipspace(p);
+
+		while (*p) {
+			char *ent;
+
+			ent = dequote(p, l = chunklen(p, check_colon(p)));
+			if (!ent) {
+				error(MODPREFIX "out of memory");
+				free(options);
+				return 1;
+			}
+
+			loc = realloc(loc, strlen(loc) + l + 2);
+			if (!loc) {
+				error(MODPREFIX "out of memory");
+				free(ent);
+				free(options);
+				return 1;
+			}
+
+			strcat(loc, " ");
+			strcat(loc, ent);
+
+			free(ent);
+
+			p += l;
+			p = skipspace(p);
+		}
+
+		loclen = strlen(loc);
 		if (loclen == 0) {
 			error(MODPREFIX "entry %s is empty!", name);
+			free(loc);
 			free(options);
 			return 1;
 		}
@@ -789,6 +1061,10 @@ int parse_mount(const char *root, const char *name,
 		      options, loclen, loc);
 
 		rv = sun_mount(root, name, name_len, loc, loclen, options);
+		/* non-strict failure to normal failure for ordinary mount */
+		if (rv < 0)
+			rv = -rv;
+			
 		free(loc);
 		free(options);
 	}
